#!/bin/bash

# some vars
#SCRIPTS_DIR=/tmp/scripts
SCRIPTS_DIR="/usr/share/lliurex-express/scripts"
INFO_DIR="$SCRIPTS_DIR/info"
MSG_STATUS_ENABLED="$(gettext "Done")"
MSG_STATUS_DISABLED="$(gettext "Unconfigured")"
MSG_STATUS_RECONFIGURE="$(gettext "Reconfigure Required")"
MSG_ERROR_CALL_PROCESS="$(gettext "Subprocess exit with error: ")"
INTERFACES_FILE="/etc/network/interfaces"

# aux funcs
die() {
        echo "$1" >&2
        exit 1
}


error_msg(){
   zenity --error --text "$1" --title "$MSG_TITLE"
}

ask_msg(){
   if zenity --question --text "$1" --title "$MSG_TITLE" ; then
      return 0
   fi
   return 1
   
}
info_msg(){
   zenity --info --text "$1" --title "$MSG_TITLE"
}

# some tests

test_root(){
   if [ $(id -u) -ne 0 ] ; then
      error_msg "$(gettext "Administration level permissions are needed to run this program")"
      exit 1
   fi
   return 0
}

get_scripts(){
   TASK_ID_LIST="$@"
   for f in $(find $INFO_DIR  -maxdepth 1 -name "???_*.meta" -type f |sort) ; do
      task_script=""
      . ${f}
      task_id="$(basename "$f" |sed -e "s%^.\{3\}_%%;s%\.meta$%%")"
      scr="${SCRIPTS_DIR}/${task_script}"
      if [ "$task_id" -a "$task_script" -a -x "${scr}" ] ; then
         if [ -z "$TASK_ID_LIST" ] || echo " $TASK_ID_LIST " |grep -q " $task_id " ; then

            echo "${scr}"
         fi
      fi
   done
}

get_task_id(){
   SRC_BASENAME="$1"
   for f in $(find $INFO_DIR  -maxdepth 1 -name "???_*.meta" -type f |sort) ; do
      task_script=""
      . ${f}
      if [ "$task_script" = "$SRC_BASENAME" ] ; then
         basename "$f" |sed -e "s%^.\{3\}_%%;s%\.meta$%%"
         return 0
      fi
   done
   return 1
}

check_prereq(){
   TASK_ID_LIST="$@"
   SCRIPT_LIST="$(get_scripts $TASK_ID_LIST)"
   for s in $SCRIPT_LIST ; do
     
     if [ -f "$s" ] ; then 
          rc=0
          $s tests || rc=$?
          if [ $rc -eq 1 ]; then
               $s message > /tmp/error.express
               return 1
          fi
     fi
   done
   return 0
}




# lib
LIB_FILE="/usr/share/lliurex/lliurex-netfuncs/lliurex-netfuncs.sh"
[ -e "$LIB_FILE" ] || exit 1
. ${LIB_FILE}

INTERFACES_FILE="/etc/network/interfaces"

srv_nic_list(){
   NETWORK_CARDS=""
   if [ $(count_network_cards) -gt 0 ] ; then
      for c in $(get_network_cards) ; do
         is_wireless_card "$c" || echo "$c"
      done
   fi
}

test_interfaces(){
   IFACE_NAME="$1"
   test_iface_static "$IFACE_NAME" "$INTERFACES_FILE" || return 1
   test_iface_auto "$IFACE_NAME" "$INTERFACES_FILE" || return 1
   return 0
}

test_interfaces_dhcp(){
	IFACE_NAME="$1"
	test_iface_dhcp "$IFACE_NAME" "$INTERFACES_FILE" || return 1
	return 0
}

test_iface_dhcp(){

	grep -q "$(iface_dhcp_regexp "$1")" $2 || return 1
	return 0

}

test_iface_static(){
   grep -q "$(iface_static_regexp "$1")" $2 || return 1
   return 0
}

test_iface_auto(){
   grep -q "^[[:blank:]]*auto\b.*\b$1\b" $2 || return 1
   return 0

}

test_iface_address(){
   test_iface_option "$1" "$2" "address" || return 1
   return 0
}

test_iface_netmask(){
   test_iface_option "$1" "$2" "netmask" || return 1
   return 0
}

test_iface_option(){
   [ "$(sed -ne "/$(iface_static_regexp "$1")/,/^[[:blank:]]*iface[[:blank:]]/{/^[[:blank:]]*$3\b/p}" "$2")" ] || return 1
   return 0
}

iface_static_regexp(){
   echo "$(iface_regexp "$1")[[:blank:]]*static\b"
}

iface_dhcp_regexp(){

	echo "$(iface_regexp "$1")[[:blank:]]*dhcp\b"
	
}

iface_regexp(){
   echo "^[[:blank:]]*iface[[:blank:]]\+$1[[:blank:]]\+inet\b"
}

test_nic(){
   [ "$1" ] || return 1
   srv_nic_list |grep -q "^${1}$" || return 1
   return 0
}

fix_interfaces_dhcp(){
   IFACE_NAME="$1"
   TMP_FILE="$(tempfile)"
   CHANGES=""

   [ ! -r "$INTERFACES_FILE" ] || cat "$INTERFACES_FILE" > "$TMP_FILE"
   if ! test_iface_auto "lo" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto lo" >> "$TMP_FILE"
   fi
   if ! grep -q "^[[:blank:]]*iface[[:blank:]]\+lo[[:blank:]]\+inet[[:blank:]]\+loopback\b" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "iface lo inet loopback" >> "$TMP_FILE"
   fi
   if ! test_iface_dhcp "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      sed -i -e "/$(iface_regexp "$IFACE_NAME")/d" "$TMP_FILE"
      echo "iface $IFACE_NAME inet dhcp" >> "$TMP_FILE"
   fi
   if ! test_iface_auto "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto $IFACE_NAME" >> "$TMP_FILE"
   fi
   if [ "$CHANGES" ] ; then
      TIMESTAMP_FILE="${INTERFACES_FILE}.$(date +%Y%m%d-%H%M%S)"
      cp -f "$INTERFACES_FILE" "$TIMESTAMP_FILE"
      llxcfg-install "$TMP_FILE" "$INTERFACES_FILE"
   fi
   rm -f "$TMP_FILE"
   return 0
}


fix_interfaces(){
   IFACE_NAME="$1"
   TMP_FILE="$(tempfile)"
   CHANGES=""
   IP="$2"
   NETMASK="$3"
   [ "$IP" ] || IP="$(get_srv_ip default)"
   [ "$NETMASK" ] || NETMASK="$(get_srv_mask default)"

   [ ! -r "$INTERFACES_FILE" ] || cat "$INTERFACES_FILE" > "$TMP_FILE"
   if ! test_iface_auto "lo" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto lo" >> "$TMP_FILE"
   fi
   if ! grep -q "^[[:blank:]]*iface[[:blank:]]\+lo[[:blank:]]\+inet[[:blank:]]\+loopback\b" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "iface lo inet loopback" >> "$TMP_FILE"
   fi
   if ! test_iface_static "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      sed -i -e "/$(iface_regexp "$IFACE_NAME")/d" "$TMP_FILE"
      echo "iface $IFACE_NAME inet static" >> "$TMP_FILE"
   fi
   if ! test_iface_auto "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto $IFACE_NAME" >> "$TMP_FILE"
   fi
   if ! test_iface_netmask "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      sed -i -e "/$(iface_static_regexp "$IFACE_NAME")/anetmask $NETMASK" "$TMP_FILE"
   fi
   if ! test_iface_address "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      sed -i -e "/$(iface_static_regexp "$IFACE_NAME")/aaddress $IP" "$TMP_FILE"
   fi
   if [ "$CHANGES" ] ; then
      TIMESTAMP_FILE="${INTERFACES_FILE}.$(date +%Y%m%d-%H%M%S)"
      cp -f "$INTERFACES_FILE" "$TIMESTAMP_FILE"
      llxcfg-install "$TMP_FILE" "$INTERFACES_FILE"
   fi
   rm -f "$TMP_FILE"
   return 0
}

set_interfaces(){
   [ "$1" ] || return 1
   TMP_FILE="$(tempfile)"
   sed -e "/$(iface_static_regexp "$1")/,/$(iface_regexp ".*")/{/^[[:blank:]]*auto\b/p;/$(iface_static_regexp "$1")/d;/$(iface_regexp ".*")/p;d}" "$INTERFACES_FILE" > "$TMP_FILE"
   llxcfg-install "$TMP_FILE" "$INTERFACES_FILE"
   rm -f "$TMP_FILE"
   fix_interfaces "$1" "$2" "$3"
   return 0
}

del_interfaces(){

	cat > $INTERFACES_FILE << EOF
#GENERATED BY LLIUREX-EXPRESS
auto lo
iface lo inet loopback

EOF
        return 0
}


